"""Геолокация"""

from __future__ import annotations

from dataclasses import dataclass
from typing import TYPE_CHECKING

from human_requests import ApiChild, ApiParent, api_child_field, autotest
from human_requests.abstraction import FetchResponse, HttpMethod

if TYPE_CHECKING:
    from ..manager import ChizhikAPI  # noqa: F401


@dataclass(init=False)
class ClassGeolocation(ApiChild["ChizhikAPI"]):
    """Методы для работы с геолокацией и выбором магазинов.

    Включает получение информации о городах, адресах, поиск магазинов
    и управление настройками доставки.
    """

    Shop: ShopService = api_child_field(lambda parent: ShopService(parent.parent))
    """Сервис для работы с информацией о магазинах."""

    def __init__(self, parent: "ChizhikAPI"):
        super().__init__(parent)
        ApiParent.__post_init__(self)

    @autotest
    async def cities_list(self, search_name: str, page: int = 1) -> FetchResponse:
        """Получить список городов по частичному совпадению имени."""
        return await self._parent._request(
            HttpMethod.GET,
            f"{self._parent.API_URL}/v1/geo/cities/?name={search_name}&page={page}",
        )


class ShopService(ApiChild["ChizhikAPI"]):
    """Сервис для работы с информацией о магазинах."""

    @autotest
    async def all(self) -> FetchResponse:
        """Получить список всех точек магазинов."""
        url = f"{self._parent.API_URL}/v1/shops"
        return await self._parent._request(HttpMethod.GET, url)

    @autotest
    async def search(self, query: str) -> FetchResponse:
        """Получить список всех точек магазинов."""
        url = f"{self._parent.API_URL}/v1/shops?term={query}"
        return await self._parent._request(HttpMethod.GET, url)



"""Реклама"""

from __future__ import annotations

from typing import TYPE_CHECKING

from human_requests import ApiChild, autotest
from human_requests.abstraction import FetchResponse, HttpMethod

if TYPE_CHECKING:
    from ..manager import ChizhikAPI  # noqa: F401


class ClassAdvertising(ApiChild["ChizhikAPI"]):
    """Методы для работы с рекламными материалами Перекрёстка.

    Включает получение баннеров, слайдеров, буклетов и другого рекламного контента.
    """

    @autotest
    async def active_inout(self) -> FetchResponse:
        """Получить активные рекламные баннеры."""
        return await self._parent._request(
            HttpMethod.GET,
            f"{self._parent.API_URL}/v1/catalog/unauthorized/active_inout/",
        )



"""Общий (не класифицируемый) функционал"""

from __future__ import annotations

from io import BytesIO
from typing import TYPE_CHECKING

from aiohttp_retry import ExponentialRetry, RetryClient
from human_requests import ApiChild
from human_requests.abstraction import Proxy

if TYPE_CHECKING:
    from ..manager import ChizhikAPI  # noqa: F401


class ClassGeneral(ApiChild["ChizhikAPI"]):
    """Общие методы API Чижика.

    Включает методы для работы с изображениями, формой обратной связи,
    получения информации о пользователе и других общих функций.
    """

    async def download_image(
        self, url: str, retry_attempts: int = 3, timeout: float = 10
    ) -> BytesIO:
        """Скачать изображение по URL."""
        retry_options = ExponentialRetry(
            attempts=retry_attempts, start_timeout=3.0, max_timeout=timeout
        )

        px = (
            self._parent.proxy
            if isinstance(self._parent.proxy, Proxy)
            else Proxy(self._parent.proxy)
        )
        async with RetryClient(retry_options=retry_options) as retry_client:
            async with retry_client.get(
                url,
                raise_for_status=True,
                proxy=px.as_str(),
            ) as resp:
                body = await resp.read()
                file = BytesIO(body)
                file.name = url.split("/")[-1]
        return file



"""Работа с каталогом"""

from __future__ import annotations

import urllib.parse
from dataclasses import dataclass
from typing import TYPE_CHECKING, Optional

from human_requests import ApiChild, ApiParent, api_child_field, autotest
from human_requests.abstraction import FetchResponse, HttpMethod

from ..abstraction import DeliveryMode

if TYPE_CHECKING:
    from ..manager import ChizhikAPI  # noqa: F401


@dataclass(init=False)
class ClassCatalog(ApiChild["ChizhikAPI"], ApiParent):
    """Методы для работы с каталогом товаров.

    Включает поиск товаров, получение информации о категориях,
    работу с фидами товаров и отзывами.
    """

    Product: ProductService = api_child_field(
        lambda parent: ProductService(parent.parent)
    )
    """Сервис для работы с товарами в каталоге."""

    def __init__(self, parent: "ChizhikAPI"):
        super().__init__(parent)
        ApiParent.__post_init__(self)

    @autotest
    async def tree(self, city_id: Optional[str] = None) -> FetchResponse:
        """Получить дерево категорий."""
        url = f"{self._parent.API_URL}/v1/catalog/unauthorized/categories/"
        if city_id:
            url += f"?city_id={city_id}"
        return await self._parent._request(HttpMethod.GET, url)

    @autotest
    async def delivery_tree(
        self,
        store_id: str,
        mode: DeliveryMode = DeliveryMode.STORE,
        include_restrict: bool = True,
    ):
        url = f"{self._parent.DELIVERY_API_URL}/catalog/v3/stores/{store_id}/categories?mode={mode}&include_subcategories=1&include_restrict={str(include_restrict).lower()}"
        return await self._parent._request(HttpMethod.GET, url)

    @autotest
    async def delivery_tree_extended(
        self,
        store_id: str,
        category_alias: str,
        mode: DeliveryMode = DeliveryMode.STORE,
        include_restrict: bool = True,
    ):
        url = f"{self._parent.DELIVERY_API_URL}/catalog/v2/stores/{store_id}/categories/{category_alias}/extended?mode={mode}&include_restrict={str(include_restrict).lower()}"
        return await self._parent._request(HttpMethod.GET, url)

    @autotest
    async def delivery_tree_ancestors(
        self,
        store_id: str,
        category_alias: str,
        mode: DeliveryMode = DeliveryMode.STORE,
        include_restrict: bool = True,
    ):
        url = f"{self._parent.DELIVERY_API_URL}/catalog/v3/stores/{store_id}/categories/{category_alias}/ancestors?mode={mode}&include_restrict={str(include_restrict).lower()}"
        return await self._parent._request(HttpMethod.GET, url)

    @autotest
    async def products_list(
        self,
        page: int = 1,
        category_id: Optional[int] = None,
        city_id: Optional[str] = None,
        search: Optional[str] = None,
    ) -> FetchResponse:
        """Получить список продуктов в категории."""
        url = f"{self._parent.API_URL}/v1/catalog/unauthorized/products/?page={page}"
        if category_id:
            url += f"&category_id={category_id}"
        if city_id:
            url += f"&city_id={city_id}"
        if search:
            url += f"&term={urllib.parse.quote(search)}"
        return await self._parent._request(HttpMethod.GET, url)

    @autotest
    async def delivery_products_list(
        self,
        store_id: str,
        category_alias: str,
        offset: int = 0,
        limit: int = 499,
        mode: DeliveryMode = DeliveryMode.STORE,
        include_restrict: bool = True,
    ):
        url = f"{self._parent.DELIVERY_API_URL}/catalog/v2/stores/{store_id}/categories/{category_alias}/products?mode={mode}&include_restrict={str(include_restrict).lower()}&limit={limit}&offset={offset}"
        return await self._parent._request(HttpMethod.GET, url)

    @autotest
    async def delivery_search(
        self,
        store_id: str,
        query: str,
        limit: int = 12,
        mode: DeliveryMode = DeliveryMode.STORE,
        include_restrict: bool = True,
    ):
        url = f"{self._parent.DELIVERY_API_URL}/catalog/v3/stores/{store_id}/search?mode={mode}&include_restrict={str(include_restrict).lower()}&q={query}&limit={limit}"
        return await self._parent._request(HttpMethod.GET, url)


class ProductService(ApiChild["ChizhikAPI"]):
    """Сервис для работы с товарами в каталоге."""

    @autotest
    async def info(
        self, product_id: int, city_id: Optional[str] = None
    ) -> FetchResponse:
        """Получить информацию о товаре по его ID.

        Args:
            product_id (int): ID товара.
            city_id (str, optional): ID города для локализации данных. Defaults to None.

        Returns:
            Response: Ответ от сервера с информацией о товаре.
        """

        url = f"{self._parent.API_URL}/v1/catalog/unauthorized/products/{product_id}/"
        if city_id:
            url += f"?city_id={city_id}"
        return await self._parent._request(HttpMethod.GET, url)

    @autotest
    async def delivery_info(
        self,
        store_id: str,
        product_id: int,
        mode: DeliveryMode = DeliveryMode.STORE,
        include_restrict: bool = True,
    ):
        # TODO
        url = f"{self._parent.DELIVERY_API_URL}/catalog/v2/stores/{store_id}/products/{product_id}?mode={mode}&include_restrict={str(include_restrict).lower()}"
        return await self._parent._request(HttpMethod.GET, url)



from __future__ import annotations

import asyncio
from collections import defaultdict
from dataclasses import dataclass, field
from typing import Any

from camoufox.async_api import AsyncCamoufox
from human_requests import (
    ApiParent,
    HumanBrowser,
    HumanContext,
    HumanPage,
    api_child_field,
)
from human_requests.abstraction import FetchResponse, HttpMethod, Proxy
from human_requests.network_analyzer.anomaly_sniffer import HeaderAnomalySniffer

from .endpoints.advertising import ClassAdvertising
from .endpoints.catalog import ClassCatalog
from .endpoints.general import ClassGeneral
from .endpoints.geolocation import ClassGeolocation


@dataclass
class ChizhikAPI(ApiParent):
    """
    Клиент Чижика.
    """

    timeout_ms: float = 10000.0
    """Время ожидания ответа от сервера в миллисекундах."""
    headless: bool = True
    """Запускать браузер в headless режиме?"""
    test_mode: bool = False
    """Режим тестирования предполагает более глубокий _warmup который не требуется для обычного использования"""
    proxy: str | dict | Proxy | None = field(default_factory=Proxy.from_env)
    """Прокси-сервер для всех запросов (если нужен). По умолчанию берет из окружения (если есть).
    Принимает как формат Playwright, так и строчный формат."""
    browser_opts: dict[str, Any] = field(default_factory=dict)
    """Дополнительные опции для браузера (см. https://camoufox.com/python/installation/)"""
    API_URL: str = "https://app.chizhik.club/api"
    DELIVERY_API_URL: str = "https://app.chizhik.club/delivery/api"
    """URL для работы с каталогом."""
    MAIN_SITE_URL: str = "https://chizhik.club/catalog/"
    """URL главной страницы сайта."""
    MAIN_SITE_ORIGIN: str = "https://chizhik.club/"

    # будет создана в __post_init__
    session: HumanBrowser = field(init=False, repr=False)
    """Внутренняя сессия браузера для выполнения HTTP-запросов."""
    # будет создано в warmup
    ctx: HumanContext = field(init=False, repr=False)
    """Внутренний контекст сессии браузера"""
    page: HumanPage = field(init=False, repr=False)
    """Внутренний страница сессии браузера"""

    unstandard_headers: dict[str, str] = field(init=False, repr=False)
    """Список нестандартных заголовков пойманных при инициализации"""
    unstandard_urls: dict[str, list[str]] = field(init=False, repr=False)
    """Список нестандартных заголовков пойманных при инициализации"""

    Geolocation: ClassGeolocation = api_child_field(ClassGeolocation)
    """API для работы с геолокацией."""
    Catalog: ClassCatalog = api_child_field(ClassCatalog)
    """API для работы с каталогом товаров."""
    Advertising: ClassAdvertising = api_child_field(ClassAdvertising)
    """API для работы с рекламой."""
    General: ClassGeneral = api_child_field(ClassGeneral)
    """API для работы с общими функциями."""

    async def __aenter__(self):
        """Вход в контекстный менеджер с автоматическим прогревом сессии."""
        await self._warmup()
        return self

    # Прогрев сессии (headless ➜ cookie `session` ➜ accessToken)
    async def _warmup(self) -> None:
        """Прогрев сессии через браузер для получения человекоподобности."""
        px = self.proxy if isinstance(self.proxy, Proxy) else Proxy(self.proxy)
        br = await AsyncCamoufox(
            headless=self.headless,
            proxy=px.as_dict(),
            **self.browser_opts,
            block_images=True,
            i_know_what_im_doing=True,
        ).start()

        self.session = HumanBrowser.replace(br)
        self.ctx = await self.session.new_context()
        self.page = await self.ctx.new_page()
        self.page.on_error_screenshot_path = "screenshot.png"

        if self.test_mode:
            sniffer = HeaderAnomalySniffer(
                include_subresources=True,  # или False, если интересны только документы
                url_filter=lambda u: u.startswith(self.API_URL),
            )
            await sniffer.start(self.ctx)

            collected = {}

            def on_request(request):
                if request.url.startswith(self.API_URL):
                    collected[request.url] = request.headers

            self.ctx.on("request", on_request)

            await self.page.goto(self.MAIN_SITE_URL, wait_until="networkidle")
            await self.page.wait_for_selector("next-route-announcer", state="attached")
            await asyncio.sleep(1)
            await self.page.locator(
                'main a[data-qa^="sidebar-sub-category-"][data-qa$="-link"]'
            ).first.click()
            await self.page.locator(
                'main div[itemtype="https://schema.org/Product"]'
            ).first.click()
            await asyncio.sleep(1)
            await self.page.wait_for_load_state("load")

            await self.ctx.unroute("**/api/**", on_request)
            result_sniffer = await sniffer.complete()

            # Результат: {заголовок: [уникальные значения]}
            result = defaultdict(set)

            # Проходим по всем URL в 'request'
            for _url, headers in result_sniffer["request"].items():
                for header, values in headers.items():
                    result[header].update(
                        values
                    )  # добавляем значения, set уберёт дубли

            # Преобразуем set обратно в list
            self.unstandard_headers = {k: list(v)[0] for k, v in result.items()}
            self.unstandard_urls = collected

        await self.page.goto(f"{self.API_URL}/v1", wait_until="networkidle")

        await self.page.wait_for_selector(
            "pre", timeout=self.timeout_ms, state="attached"
        )

    async def __aexit__(self, *exc):
        """Выход из контекстного менеджера с закрытием сессии."""
        await self.close()

    async def close(self):
        """Закрыть HTTP-сессию и освободить ресурсы."""
        await self.session.close()

    async def _request(
        self,
        method: HttpMethod,
        url: str,
        *,
        json_body: Any | None = None,
    ) -> FetchResponse:
        """Выполнить HTTP-запрос через внутреннюю сессию.

        Единая точка входа для всех HTTP-запросов библиотеки.
        Добавляет к ответу объект Request для совместимости.

        Args:
            method: HTTP метод (GET, POST, PUT, DELETE и т.д.)
            url: URL для запроса
            json_body: Тело запроса в формате JSON (опционально)
        """
        resp: FetchResponse = await self.page.fetch(
            url=url,
            method=method,
            body=json_body,
            credentials="same-origin",
            mode="cors",
            timeout_ms=self.timeout_ms,
            referrer=self.MAIN_SITE_ORIGIN,
            headers={"Accept": "application/json, text/plain, */*"},
        )

        return resp
